/*
 * AccessorBinding.java
 *
 * Copyright (C) 2008-2009 Alessio Stalla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

package snow.binding;

import org.armedbear.lisp.JavaObject;
import org.armedbear.lisp.LispObject;

import com.jgoodies.binding.value.AbstractValueModel;
import com.jgoodies.binding.value.ValueModel;

public class AccessorBinding extends AbstractValueModel implements ValueModel {

	private LispObject reader;
	private LispObject writer;
	private LispObject place;
	private Object oldValue; 
	
	public AccessorBinding(LispObject place, LispObject reader, LispObject writer) {
		super();
		this.place = place;
		this.reader = reader;
		this.writer = writer;
		oldValue = getValue();
	}
	
	
    @Override
    public Object getValue() {
	return reader.execute(place).javaInstance();
    }

    @Override
    public void setValue(Object value) {
	writer.execute(JavaObject.getInstance(value, true), place);
    }
	
    /**
     * Called from Lisp to notify a value change without invoking the writer.
     */
    public void valueChanged(Object value) {
	fireValueChange(oldValue, value, false);
	oldValue = value;
    }
    
    public LispObject getPlace() {
	return place;
    }


	public void setPlace(LispObject place) {
		this.place = place;
	}

	
	public LispObject getReader() {
		return reader;
	}
	
	public void setReader(LispObject reader) {
		this.reader = reader;
	}
	
	public LispObject getWriter() {
		return writer;
	}
	
	public void setWriter(LispObject writer) {
		this.writer = writer;
	}

}
