/*
 * WindowListener.java
 *
 * Copyright (C) 2008-2009 Alessio Stalla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

package snow.swing;

import java.awt.event.MouseEvent;

import org.armedbear.lisp.JavaObject;
import org.armedbear.lisp.LispObject;

public class MouseInputListener implements javax.swing.event.MouseInputListener {

    private LispObject mouseClicked, mouseEntered, mouseExited, mousePressed, mouseReleased;
    private LispObject mouseDragged, mouseMoved;
	
    public MouseInputListener(LispObject mouseClicked, LispObject mousePressed, LispObject mouseReleased, LispObject mouseEntered, LispObject mouseExited, LispObject mouseDragged, LispObject mouseMoved) {
	super();
	this.mouseClicked = mouseClicked;
	this.mousePressed = mousePressed;
	this.mouseReleased = mouseReleased;

	this.mouseEntered = mouseEntered;
	this.mouseExited = mouseExited;

	this.mouseDragged = mouseDragged;
	this.mouseMoved = mouseMoved;
    }
	
    private static final void invokeDelegate(LispObject delegate, MouseEvent e) {
	if(delegate != null) {
	    delegate.execute(new JavaObject(e));
	}
    }
	
    public void mouseClicked(MouseEvent e) {
	invokeDelegate(mouseClicked, e);
    }

    public void mouseEntered(MouseEvent e) {
	invokeDelegate(mouseEntered, e);
    }

    public void mouseExited(MouseEvent e) {
	invokeDelegate(mouseExited, e);
    }

    public void mousePressed(MouseEvent e) {
	invokeDelegate(mousePressed, e);
    }

    public void mouseReleased(MouseEvent e) {
	invokeDelegate(mouseReleased, e);
    }

    public void mouseDragged(MouseEvent e) {
	invokeDelegate(mouseDragged, e);
    }

    public void mouseMoved(MouseEvent e) {
	invokeDelegate(mouseMoved, e);
    }

}
